/***************************************************************************
 *   This file is part of the eXaro project                                *
 *   Copyright (C) 2008 by BogDan Vatra                                    *
 *   bog_dan_ro@yahoo.com                                                  *
 **                   GNU General Public License Usage                    **
 *                                                                         *
 *   This library is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 **                  GNU Lesser General Public License                    **
 *                                                                         *
 *   This library is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        * 
 *   published by the Free Software Foundation, either version 3 of the    *
 *   License, or (at your option) any later version.                       *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library.                                      * 
 *   If not, see <http://www.gnu.org/licenses/>.                           *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 ****************************************************************************/

#ifndef REPORTPAINTDEVICE_H
#define REPORTPAINTDEVICE_H

#include <QPaintDevice>
#include <QBuffer>
#include <QDomDocument>
#include <QPrinter>

namespace Report
{

class PaintEngine;

class PaintDevice : public QPaintDevice
{
public:
	struct PageStruct
	{
		unsigned size;
		qreal width,height;
		QPrinter::Orientation orientation;
	};

private:
	friend class PaintEngine;

public:
	PaintDevice(QIODevice * doc);

	~PaintDevice();
	void setPaperSize(const QSizeF & pageSize);
	void setPaperOrientation(QPrinter::Orientation orientation);
	void newPage();
	void allowEmptyPages(bool allow);
	QPaintEngine * paintEngine() const;
	int metric(QPaintDevice::PaintDeviceMetric metric) const;
	void setPrinterName(const QString & name);

private:
	QIODevice * m_doc;
	bool m_allowEmptyPages;
	PaintEngine * m_paintEngine;
	qint64 m_pagePos;
	PageStruct m_currentPageStruct;
	QPrinter m_printer;
};

}

#endif
